 #!/usr/local/bin/perl -wl
use strict;
use warnings;

eval '
    no drugs;
    use Life;
';

open my $heart => "/dev/null" and print "yeah";

package slackage;
my $baggage = <<'FOREVER'
    fun!
FOREVER
